from erp_sync.Resources.resource import Resource


class Payments(Resource):

    urls = {}

    def set_client_id(self, client_id):
        super().set_client_id(client_id)
        self._set_urls()
        return self

    def set_company_id(self, company_id):
        super().set_company_id(company_id)
        self._set_urls()
        return self

    def _set_urls(self):

        self.urls = {
            "new": f"/companies/{super().get_company_id()}/payments",
            "edit": f"/companies/{super().get_company_id()}/payments",
            "delete": f"/companies/{super().get_company_id()}/payments",
            "read": f"/companies/{super().get_company_id()}/payments",
            "import": f"/companies/{super().get_company_id()}/import_payments"
        }

        super().set_urls(self.urls)

        return self

    def read(self, payment_id=None, payload=None, method='GET', endpoint=None):

        self._set_urls()

        if payment_id is not None:
            self.urls["read"] = f'{self.urls["read"]}/{payment_id}'
            super().set_urls(self.urls)

        return super().read(payload, method, endpoint)

    def edit(self, ledger_id=None, payload=None, method='PUT', endpoint=None):

        self._set_urls()

        self.urls["edit"] = f'{self.urls["edit"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().edit(payload, method, endpoint)

    def delete(self, ledger_id=None, payload=None, method='DELETE', endpoint=None):

        payload = {"type": "SalesPayment"}

        self._set_urls()

        self.urls["delete"] = f'{self.urls["delete"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().delete(payload, method, endpoint)

    def import_data(self, ledger_id=None, payload=None, method='GET', endpoint=None):

        self._set_urls()

        if ledger_id is not None:
            self.urls["import"] = f'{self.urls["import"]}/{ledger_id}'
            super().set_urls(self.urls)

        return super().import_data(payload, method, endpoint)

    def payload(self):

        data = {
            "amount": "<Enter amount>",
            "customer_id": "<Enter customer id>",
            "reference": "<Enter unique reference>",
            "description": "<Enter description>",
            "date": "<Enter date (yyyy-mm-dd) e.g. 2021-11-22>"
        }

        # If client type is ZOHO
        if super().get_client_type() == super().XERO:
            data["customer_id"] = "<Chart of Account ID>"
            data["invoice_id"] = "<Enter invoice id>"

        return data

    def serialize(self, payload=None, operation=None):

        data = {}

        if operation is None:
            return "Specify the operation: Resource.READ, Resource.NEW or Resource.UPDATE"

        if operation == super().NEW or operation == super().UPDATE:

            data["type"] = "SalesPayment"

            if 'type' in payload.keys():
                data["type"] = payload.get("type", "SalesPayment")

            additional_properties = payload.get("additional_properties", {})

            # If client type is Quickbooks Online
            if super().get_client_type() == super().QBO:

                if 'amount' in payload.keys():
                    data.update({
                        "TotalAmt": round(float(payload.get("amount", 0)),2)
                    })

                if 'customer_id' in payload.keys():
                    data.update({
                        "CustomerRef": {
                            "value": payload.get("customer_id", "")
                        }
                    })

                if 'currency_code' in payload.keys():
                    data.update({
                        "CurrencyRef": {
                            "value": payload.get("currency_code", "KES")
                        }
                    })
                
                invoices = payload.get("invoice_payments", [])

                for i in range(len(invoices)): 
                    if i == 0:
                        if 'payment_reference' in invoices[i].keys():
                            data.update({
                                "PaymentRefNum": invoices[i].pop('payment_reference')
                            })
                    
                    if 'payment_date' in invoices[i].keys():
                        invoices[i].pop('payment_date')
                    if 'payment_type' in invoices[i].keys():
                        invoices[i].pop('payment_type')
                    if 'account_id' in invoices[i].keys():
                        invoices[i].pop('account_id')

                    invoice = {}
                    if 'amount' in invoices[i].keys():
                        invoice.update({
                            "Amount": round(float(invoices[i].get("amount", 0)),2)
                        })

                    if 'invoice_id' in invoices[i].keys():
                        invoice.update({
                            "LinkedTxn": [
                                {
                                    "TxnId": invoices[i].get('invoice_id',""),
                                    "TxnType": "Invoice"
                                }
                            ]
                        })
                    
                    invoices[i] = invoice

                # if invoices has data in it
                if bool(invoices):
                    data.update({
                        "Line": invoices
                    })

            # If client type is ZOHO
            elif super().get_client_type() == super().ZOHO:

                payment = {
                    "payment_mode": payload.get("payment_mode", "cash")
                }

                if 'customer_id' in payload.keys():
                    payment.update({
                        "customer_id": payload.get("customer_id", "")
                    })

                if 'amount' in payload.keys():
                    payment.update({
                        "amount": f'{round(float(payload.get("amount", 0)),2)}'
                    })

                # if 'reference' in payload.keys():
                #     payment.update({
                #         "reference_number": payload.get("reference", "")
                #     })

                if 'description' in payload.keys():
                    payment.update({
                        "description": payload.get("description", "")
                    })

                if 'date' in payload.keys():
                    payment.update({
                        "date": payload.get("date", "")
                    })

                invoices = payload.get("invoice_payments", [])

                for i in range(len(invoices)): 
                    if i == 0:
                        if 'payment_reference' in invoices[i].keys():
                            payment.update({
                                "reference_number": invoices[i].pop('payment_reference')
                            })
                    if 'invoice_id' in invoices[i].keys():
                        invoices[i]['invoice_id'] = invoices[i].pop(
                            'invoice_id')
                    if 'amount' in invoices[i].keys():
                        invoices[i]['amount_applied'] = invoices[i].pop(
                            'amount')
                            
                    if 'payment_date' in invoices[i].keys():
                        invoices[i].pop('payment_date')
                    if 'payment_type' in invoices[i].keys():
                        invoices[i].pop('payment_type')
                    if 'account_id' in invoices[i].keys():
                        invoices[i].pop('account_id')
                # if invoices has data in it
                if bool(invoices):
                    payment.update({
                        "invoices": invoices
                    })

                if bool(payment):
                    data.update({
                        "payment": payment
                    })

            # If client type is SAP
            elif super().get_client_type() == super().SAP:
                header = {}

                if 'type' in data.keys():
                    header.update({
                        "Type": data.pop("type")
                    })                

                if 'date' in payload.keys():
                    header.update({
                        "DocDate": payload.get("date", "")
                    })

                if 'customer_id' in payload.keys():
                    header.update({
                        "CardCode": payload.get("customer_id", "")
                    })

                if 'invoice_id' in payload.keys():
                    header.update({
                        "InvoiceDocEntry": payload.get("invoice_id", "")
                    })
                
                if 'SourceNumber' in additional_properties.keys():
                    header.update({
                        "SourceNumber": additional_properties.get("SourceNumber", "")
                    })
                    
                    additional_properties.pop("SourceNumber")
                
                if 'PostingDate' in additional_properties.keys():
                    header.update({
                        "PostingDate": additional_properties.get("PostingDate", "")
                    })
                    
                    additional_properties.pop("PostingDate")
                
                if 'DocCurrency' in additional_properties.keys():
                    header.update({
                        "DocCurrency": additional_properties.get("DocCurrency", "")
                    })
                    
                    additional_properties.pop("DocCurrency")
                
                if 'CardType' in additional_properties.keys():
                    header.update({
                        "CardType": additional_properties.get("CardType", "")
                    })
                    
                    additional_properties.pop("CardType")
                
                if 'ReceiptNo' in additional_properties.keys():
                    header.update({
                        "ReceiptNo": additional_properties.get("ReceiptNo", "")
                    })
                    
                    additional_properties.pop("ReceiptNo")
                
                invoice_payments = payload.get("invoice_payments", [])

                for i in range(len(invoice_payments)):
                    if 'payment_date' in invoice_payments[i].keys():
                        invoice_payments[i]['PaymentDate'] = invoice_payments[i].pop(
                            'payment_date')
                    if 'payment_reference' in invoice_payments[i].keys():
                        invoice_payments[i]['PaymentReference'] = invoice_payments[i].pop(
                            'payment_reference')
                    if 'payment_type' in invoice_payments[i].keys():
                        invoice_payments[i]['PaymentType'] = invoice_payments[i].pop(
                            'payment_type')
                    if 'account_id' in invoice_payments[i].keys():
                        invoice_payments[i]['Account'] = invoice_payments[i].pop(
                            'account_id')
                    if 'amount' in invoice_payments[i].keys():
                        invoice_payments[i]['Amount'] = int(float(invoice_payments[i].pop('amount')))
                    
                    if 'invoice_id' in invoice_payments[i].keys():
                        invoice_payments[i].pop('invoice_id')

                # if invoice_payments has data in it
                if bool(invoice_payments):
                    header.update({
                        "Payments": invoice_payments
                    })
                
                # if header has data in it
                if bool(header):
                    data.update({
                        "Header": header
                    })

            # If client type is ZOHO
            elif super().get_client_type() == super().XERO:

                data = {}

                payment = {}

                if 'date' in payload.keys():
                    payment.update({
                        "Date": payload.get("date", "")
                    })
                
                invoice = payload.get("invoice_payments", [])

                if bool(invoice):
                    if "account_id" in invoice[0].keys():
                        payment.update({
                            "Account": {
                                "AccountID": f'{invoice[0].pop("account_id")}',
                            }
                        })
                    if "invoice_id" in invoice[0].keys():
                        payment.update({
                            "Invoice": {
                                "InvoiceID": invoice[0].pop("invoice_id"),
                            }
                        }) 
                    if 'amount' in invoice[0].keys():
                        payment.update({
                            "Amount": int(float(invoice[0].pop('amount')))
                        })
                    if 'payment_reference' in invoice[0].keys():
                        payment.update({
                            "Reference": invoice[0].pop('payment_reference')
                        }) 
                    
                    if 'payment_date' in invoice[0].keys():
                        invoice[0].pop('payment_date')
                    if 'payment_type' in invoice[0].keys():
                        invoice[0].pop('payment_type')

                if bool(payment):
                    data.update({
                        "Payments": [payment]
                    })

            # If client type is ODOO
            elif super().get_client_type() == super().ODOO:
                data = {
                    "partner_type": "customer",
                    "date": payload.get("date", ""),
                    "amount": round(float(payload.get("amount", 0)),2),
                    # "partner_id": int(float(payload.get("customer_id", 0))),
                    # "currency_code": payload.get("currency_code", ""),
                    # "currency_id": int(payload.get("currency_id", 0)),
                    # "journal_id": int(payload.get("account_id", 0)),
                    "state": payload.get("state", "draft"),
                }

                if 'customer_id' in payload.keys():
                    if payload.get("customer_id")!='':
                        data['partner_id']=int(float(payload.get("customer_id")))

                if 'currency_code' in payload.keys():
                    if payload.get("currency_code")!='':
                        data['currency_code']=payload.get("currency_code", "")

                invoices = payload.get("invoice_payments", [])

                if invoices is not None:

                    for i in range(len(invoices)):                    
                        if i == 0:
                            if 'payment_reference' in invoices[i].keys():
                                data['payment_reference'] = invoices[i].pop('payment_reference')

                        if "invoice_id" in invoices[i].keys():
                            invoices[i]["invoice_id"] = invoices[i].pop("invoice_id")
                        if "amount" in invoices[i].keys():
                            invoices[i]["amount"] = round(float(invoices[i].pop("amount")),2)

                        if 'payment_date' in invoices[i].keys():
                            invoices[i].pop('payment_date')
                        if 'payment_type' in invoices[i].keys():
                            invoices[i].pop('payment_type')
                        if 'account_id' in invoices[i].keys():
                            invoices[i].pop('account_id')

                    # if line_items has data in it
                    if bool(invoices):
                        data.update({"invoices": invoices})

            # If client type is MS_DYNAMICS
            elif super().get_client_type() == super().MS_DYNAMICS:
                data.pop("type")
                data.update({
                    # "paymentReference": payload.get("reference", ""),
                    "ownerInvoiceType": "Customer",
                    # "ownerInvoiceNumber": f'{payload.get("invoice_id", "")}',
                    # "ownerNumber": payload.get("customer_id", ""),
                    "description": payload.get("description", ""),
                    # "amount": payload.get("amount", 0),
                    "bankCode": payload.get("bank_code", ""),
                })

                if 'customer_id' in payload.keys():
                    if payload.get("customer_id")!='':
                        data['ownerNumber']=payload.get("customer_id")

                invoice = payload.get("invoice_payments", [])

                if bool(invoice):
                    if "invoice_id" in invoice[0].keys():
                        data.update({
                            "ownerInvoiceNumber": invoice[0].pop("invoice_id"),
                        }) 
                    if 'payment_reference' in invoice[0].keys():
                        data.update({
                            "paymentReference": invoice[0].pop('payment_reference')
                        })
                    if 'amount' in invoice[0].keys():
                        data.update({
                            "amount": round(float(invoice[0].pop('amount')),2)
                        })                    
                    
                    if 'payment_date' in invoice[0].keys():
                        invoice[0].pop('payment_date')
                    if 'payment_type' in invoice[0].keys():
                        invoice[0].pop('payment_type')
                    if 'account_id' in invoice[0].keys():
                        invoice[0].pop('account_id')

            data.update(additional_properties)

            return data

        elif operation == super().READ:

            payload = super().response()

            if 'data' in payload.keys():
                payload = payload.get("data", [])

            elif 'resource' in payload.keys():
                payload = payload.get("resource", [])

            # confirms if a single object was read from the database
            if isinstance(payload, dict):
                payload = [payload]

            if payload is not None:
                
                if len(payload) > 0:
                    for i in range(len(payload)):
                        if 'chart_of_account_id' in payload[i].keys():
                            payload[i]['customer_id'] = payload[i].pop(
                                'chart_of_account_id')
                        if 'reference_number' in payload[i].keys():
                            payload[i]['reference'] = payload[i].pop(
                                'reference_number')
                        if 'total_amount' in payload[i].keys():
                            payload[i]['amount'] = payload[i].pop('total_amount')
                
            else:
                payload = super().response()

            super().set_response(payload)

            return self



from erp_sync.Resources.clients import Client
from erp_sync.Resources.users import Users
from erp_sync.Resources.resource import Resource

# This class is responsible for configuring and checking of developer credentials.
# It's the class that will be called first before a developer accesses Nash Framework - Nash's API resources

# This class is also a Resource class
class Nash(Resource):
    # Since all classes are going to be called from this class,
    # You can use this class to set attributes of other classes/resources
    _headers = {
        'Content-Type': 'application/json',
        'Accept-Encoding': 'gzip, deflate, br',
    }

    _params = {}

    _response = {}

    _is_logged_in = False

    _client = None

    _access_token = None

    _user_id = -1

    # When intializing this class, set the default request headers and params
    def __init__(self):
        super().__init__("NashAPI", self._headers, self._params)

    # Use this method to login a user into the nash framework
    def login(self, payload=None, method='POST', endpoint="/auth/login"):
        # authenticate and provide user credentials
        # set response here

        # Call the method exec to make the request to A.P.I. endpoint and return the data that will be returned in this method
        self._response = Users(self).login(
            payload, method, endpoint).response()

        if self._response["status"] == 200:

            super().set_user_id(self._response["id"])

            if 'access_token' in self._response:

                self._set_access_token(self._response["access_token"])
                self._is_logged_in = True

                self._headers["Authorization"] = f"Bearer {self.get_access_token()}"

        return self

    # Use this method to sign up a user into the nash framework
    def sign_up(self, payload=None, method='POST', endpoint="/users", log_in_user=True):
        # authenticate and provide user credentials
        # set response here

        # Call the method exec to make the request to A.P.I. endpoint and return the data that will be returned in this method
        self._response = Users(self).new(payload, method, endpoint).response()

        if log_in_user:
            if 'created_at' in self._response:

                self.login(payload={"username": payload.get(
                    "username"), "password": payload.get("password")}).response()

        return self

    # Use this method to get access to the ERP/Client class, this is the gateway to the ERP Resources/Entities and 
    def client(self, client_id=-1):

        if self._client is None:
            self._client = Client(self, client_id)

        return self._client

    def user(self):
        return Users(self)

    # Method to check if a user is logged in
    def is_logged_in(self):
        return self._is_logged_in

    # Method to set user log in access token
    def _set_access_token(self, access_token):
        self._access_token = access_token
        return self

    # Method to get user log in access token
    def get_access_token(self):
        return self._access_token



from erp_sync.Resources.resource import Resource
from datetime import datetime, timedelta

class Invoices(Resource):

    urls = {}

    def set_client_id(self, client_id):
        super().set_client_id(client_id)
        self._set_urls()
        return self

    def set_company_id(self, company_id):
        super().set_company_id(company_id)
        self._set_urls()
        return self

    def _set_urls(self):

        self.urls = {
            "new": f"/companies/{super().get_company_id()}/invoices",
            "edit": f"/companies/{super().get_company_id()}/invoices",
            "read": f"/companies/{super().get_company_id()}/invoices",
            "delete": f"/companies/{super().get_company_id()}/invoices",
            "import": f"/companies/{super().get_company_id()}/import_invoices",
        }

        super().set_urls(self.urls)

        return self

    def edit(self, ledger_id=None, payload=None, method="PUT", endpoint=None):

        self._set_urls()

        self.urls["edit"] = f'{self.urls["edit"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().edit(payload, method, endpoint)

    def read(self, invoice_id=None, payload=None, method="GET", endpoint=None):

        self._set_urls()

        if invoice_id is not None:
            self.urls["read"] = f'{self.urls["read"]}/{invoice_id}'
            super().set_urls(self.urls)

        return super().read(payload, method, endpoint)

    def delete(self, ledger_id=None, payload=None, method="DELETE", endpoint=None):

        self._set_urls()

        # payload = {"type": "SalesInvoice"}

        self.urls["delete"] = f'/{self.urls["delete"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().delete(payload, method, endpoint)

    def import_data(self, ledger_id=None, date_from = None, payload=None, method="GET", endpoint=None):

        self._set_urls()

        if ledger_id is not None:
            self.urls["import"] = f'{self.urls["import"]}/{ledger_id}'
            super().set_urls(self.urls)        
        
        if date_from is not None and date_from != '':
            self.urls["import"] = f'{self.urls["import"]}?dateFromFilter={date_from}'
            super().set_urls(self.urls)

        return super().import_data(payload, method, endpoint)

    def apply_credits(self, ledger_id=None, payload=None, method="POST", endpoint=None):

        endpoint = f'{self.urls["read"]}/{ledger_id}/apply_credits'

        return super().read(payload, method, endpoint)

    def invoice_payments(
        self, invoice_id=None, payload=None, method="GET", endpoint=None
    ):

        endpoint = f'{self.urls["read"]}/{invoice_id}/payments'

        return super().read(payload, method, endpoint)

    def credits_applied(
        self, invoice_id=None, payload=None, method="GET", endpoint=None
    ):

        endpoint = f'{self.urls["read"]}/{invoice_id}/credits_applied'

        return super().read(payload, method, endpoint)

    def invoice_types(self, endpoint="/invoice_types"):

        return super().read(endpoint=endpoint)

    def import_bills(self, invoice_id=None, payload=None, method="GET", endpoint=None):
        endpoint = f"/companies/{super().get_company_id()}/import_bills"

        if invoice_id is not None:
            endpoint = f"{endpoint}/{invoice_id}"

        return super().read(payload, method, endpoint)

    def payload(self):

        data = {
            "customer_ledger_id ": "<Enter customer id>",
            "item_ledger_id": "<Enter item id>",
            "amount": "<Enter amount>",
            "reference": "<reference>",
            "due_date": "<Enter invoice due date>",
        }

        # If client type is XERO
        if super().get_client_type() == super().XERO:
            data["chart_of_account_id"] = "<Enter chart of account id>"

        # If client type is ZOHO
        elif (
            super().get_client_type() == super().ZOHO
            or super().get_client_type() == super().QBO
        ):
            data.pop("reference")
            data.pop("due_date")

        return data

    def serialize(self, payload=None, operation=None):

        data = {}

        if operation is None:
            return (
                "Specify the operation: Resource.READ, Resource.NEW or Resource.UPDATE"
            )

        if operation == super().NEW or operation == super().UPDATE:

            data["type"] = "SalesInvoice"

            if "invoice_type" in payload.keys():
                data["type"] = payload.get("invoice_type", "SalesInvoice")

            additional_properties = payload.get("additional_properties", {})

            # If client type is ZOHO
            if super().get_client_type() == super().ZOHO:

                if "customer_ledger_id" in payload.keys():
                    data.update({"customer_id": payload.get("customer_ledger_id", "")})

                if "reference" in payload.keys():
                    data.update({"reference_number": payload.get("reference", "")})

                if "invoice_number" in payload.keys():
                    data.update({"invoice_number": payload.get("invoice_number", "")})

                data.update({"due_date": payload.get("due_date", (datetime.now() + timedelta(days=1)).strftime("%Y-%m-%d"))})
                data.update({"date": payload.get("invoice_date", datetime.now().strftime("%Y-%m-%d"))})

                line_items = payload.get("line_items", [])

                for i in range(len(line_items)):
                    if "item_ledger_id" in line_items[i].keys():
                        line_items[i]["item_id"] = line_items[i].pop("item_ledger_id")

                    if "price" in line_items[i].keys():
                        line_items[i]["rate"] = line_items[i].pop("price")

                    if "quantity" in line_items[i].keys():
                        line_items[i]["quantity"] = line_items[i].pop("quantity")

                    if "account_id" in line_items[i].keys():
                        line_items[i].pop("account_id")
                    if "type" in line_items[i].keys():
                        line_items[i].pop("type")
                    if "unit_of_measure" in line_items[i].keys():
                        line_items[i].pop("unit_of_measure")
                    if "tax_type" in line_items[i].keys():
                        line_items[i].pop("tax_type")
                    if "tax_id" in line_items[i].keys():
                        line_items[i].pop("tax_id")
                    if "unit_of_measure" in line_items[i].keys():
                        line_items[i].pop("unit_of_measure")

                # if line_items has data in it
                if bool(line_items):
                    data.update({"line_items": line_items})

            # If client type is Quickbooks Online
            elif super().get_client_type() == super().QBO:

                if "customer_ledger_id" in payload.keys():
                    data.update(
                        {
                            "CustomerRef": {
                                "value": payload.get("customer_ledger_id", "")
                            }
                        }
                    )

                if "currency_code" in payload.keys():
                    data.update({
                        "CurrencyRef": {
                            "value": payload.get("currency_code", "KES")
                        }
                    })

                line_items = payload.get("line_items", [])

                for i in range(len(line_items)):
                    line_items[i]["DetailType"] = "SalesItemLineDetail"
                    line_items[i]["Description"] = line_items[i].pop("description")
                    line_items[i]["SalesItemLineDetail"] = {}
                    line_items[i]["Amount"] = line_items[i].get(
                        "price", 0
                    ) * line_items[i].get("quantity", 0)

                    if "item_ledger_id" in line_items[i].keys():
                        line_items[i]["SalesItemLineDetail"]["ItemRef"] = {"value": line_items[i].pop("item_ledger_id", 0)}

                    if "price" in line_items[i].keys():
                        line_items[i]["SalesItemLineDetail"]["UnitPrice"] = line_items[i].pop("price")

                    if "quantity" in line_items[i].keys():
                        line_items[i]["SalesItemLineDetail"]["Qty"] = line_items[i].pop("quantity")
                    
                    if "account_id" in line_items[i].keys():
                        line_items[i].pop("account_id")
                    if "type" in line_items[i].keys():
                        line_items[i].pop("type")
                    if "unit_of_measure" in line_items[i].keys():
                        line_items[i].pop("unit_of_measure")
                    if "tax_type" in line_items[i].keys():
                        line_items[i].pop("tax_type")
                    if "tax_id" in line_items[i].keys():
                        line_items[i].pop("tax_id")

                # if line_items has data in it
                if bool(line_items):
                    data.update({"Line": line_items})

            # If client type is SAP
            elif super().get_client_type() == super().SAP:
                header = {}

                if "customer_ledger_id" in payload.keys():
                    header.update({"CardCode": payload.get("customer_ledger_id", "")})

                if "invoice_type" in payload.keys():
                    header.update(
                        {
                            "ObjectType": data.get("type", ""),
                            "InvoiceType": data.get("type", ""),
                        }
                    )

                header.update({"PostingDate": payload.get("invoice_date", datetime.now().strftime("%Y-%m-%d"))})
                header.update({"DocDate": payload.get("invoice_date", datetime.now().strftime("%Y-%m-%d"))})
                header.update({"DocDueDate": payload.get("due_date", (datetime.now() + timedelta(days=1)).strftime("%Y-%m-%d"))})

                if "reference" in payload.keys():
                    header.update({"Reference": payload.get("reference", "")})

                if "DocDate" in additional_properties.keys():
                    header.update({"DocDate": additional_properties.get("DocDate", "")})

                    additional_properties.pop("DocDate")

                if "PostingDate" in additional_properties.keys():
                    header.update(
                        {"PostingDate": additional_properties.get("PostingDate", "")}
                    )

                    additional_properties.pop("PostingDate")

                if "RequiredDate" in additional_properties.keys():
                    header.update(
                        {"RequiredDate": additional_properties.get("RequiredDate", "")}
                    )

                    additional_properties.pop("RequiredDate")

                if "ValidUntil" in additional_properties.keys():
                    header.update(
                        {"ValidUntil": additional_properties.get("ValidUntil", "")}
                    )

                    additional_properties.pop("ValidUntil")

                if "DocCurrency" in additional_properties.keys():
                    header.update(
                        {"DocCurrency": additional_properties.get("DocCurrency", "")}
                    )

                    additional_properties.pop("DocCurrency")

                if "SourceNumber" in additional_properties.keys():
                    header.update(
                        {"SourceNumber": additional_properties.get("SourceNumber", "")}
                    )

                    additional_properties.pop("SourceNumber")

                if "Rounding" in additional_properties.keys():
                    header.update(
                        {"Rounding": additional_properties.get("Rounding", "")}
                    )

                    additional_properties.pop("Rounding")

                if "Remarks" in additional_properties.keys():
                    header.update({"Remarks": additional_properties.get("Remarks", "")})

                    additional_properties.pop("Remarks")

                line_items = payload.get("line_items", [])

                for i in range(len(line_items)):
                    if "account_id" in line_items[i].keys():
                        line_items[i]["AcctCode"] = line_items[i].pop("account_id")
                    if "item_ledger_id" in line_items[i].keys():
                        line_items[i]["ItemCode"] = line_items[i].pop("item_ledger_id")
                    if "quantity" in line_items[i].keys():
                        line_items[i]["Quantity"] = line_items[i].pop("quantity")
                    if "price" in line_items[i].keys():
                        line_items[i]["UnitPrice"] = line_items[i].pop("price")
                    if "description" in line_items[i].keys():
                        line_items[i]["Description"] = line_items[i].pop("description")
                    if "tax_type" in line_items[i].keys():
                        line_items[i]["VatGroup"] = line_items[i].pop("tax_type")

                # if line_items has data in it
                if bool(line_items):
                    header.update({"Rows": line_items})

                # if header has data in it
                if bool(header):
                    data.update({"Header": header})

            # If client type is XERO
            elif super().get_client_type() == super().XERO:

                invoices = {
                    "Type": "ACCREC",
                    "Status": f'{additional_properties.get("Status", "DRAFT")}',
                    "Date": payload.get("invoice_date", datetime.now().strftime("%Y-%m-%d")),
                    "DueDate": payload.get("due_date", (datetime.now() + timedelta(days=1)).strftime("%Y-%m-%d"))
                }

                if "currency_code" in payload.keys():
                    invoices.update(
                        {"CurrencyCode": payload.get("currency_code", "KES")}
                    )

                if "customer_ledger_id" in payload.keys():
                    invoices.update(
                        {
                            "Contact": {
                                "ContactID": payload.get("customer_ledger_id", "")
                            }
                        }
                    )

                line_items = payload.get("line_items", [])

                for i in range(len(line_items)):
                    if "tax_type" in line_items[i].keys():
                        line_items[i]["TaxType"] = line_items[i].pop("tax_type")
                    elif "tax_type" not in line_items[i].keys():
                        line_items[i]["TaxType"] = "NONE"

                    if "price" in line_items[i].keys():
                        line_items[i]["UnitAmount"] = int(float(line_items[i].pop("price")))

                    if "quantity" in line_items[i].keys():
                        line_items[i]["Quantity"] = int(float(line_items[i].pop("quantity")))

                    line_items[i]["LineAmount"] = line_items[i].get("UnitAmount",0) * line_items[i].get("Quantity",0)

                    if "description" in line_items[i].keys():
                        line_items[i]["Description"] = line_items[i].pop("description")

                    if "account_id" in line_items[i].keys():
                        line_items[i]["AccountID"] = line_items[i].pop("account_id")

                    if "item_ledger_id" in line_items[i].keys():
                        line_items[i]["LineItemID"] = line_items[i].pop(
                            "item_ledger_id"
                        )

                    if "tax_id" in line_items[i].keys():
                        line_items[i].pop("tax_id")
                    if "type" in line_items[i].keys():
                        line_items[i].pop("type")
                    if "unit_of_measure" in line_items[i].keys():
                        line_items[i].pop("unit_of_measure")
                    

                if "reference" in payload.keys():
                    invoices.update({"Reference": payload.get("reference", "")})

                if "Status" in additional_properties.keys():
                    additional_properties.pop("Status")
                

                # if line_items has data in it
                if bool(line_items):
                    invoices.update({"LineItems": line_items})

                # if invoices has data in it
                if bool(invoices):
                    data.update({"Invoices": [invoices]})

            # If client type is ODOO
            elif super().get_client_type() == super().ODOO:
                data.update(
                    {
                        "partner_id": int(payload.get("customer_ledger_id", 0)),
                        "name": payload.get("invoice_number", ""),
                        "payment_reference": payload.get("reference", ""),
                        "state": additional_properties.get("state", "draft"),
                        "currency_id": int(payload.get("currency_id", 0)),
                        "currency_code": payload.get("currency_code", ""),
                        "invoice_date": payload.get("invoice_date", datetime.now().strftime("%Y-%m-%d")),
                        "invoice_date_due": payload.get("due_date", (datetime.now() + timedelta(days=1)).strftime("%Y-%m-%d")),
                    }
                )

                if data["type"] == "SalesInvoice":
                    data.update({"move_type": "out_invoice"})
                    data.pop("type")
                elif data["type"] == "PurchaseInvoice":
                    data.update({"move_type": "in_invoice"})
                    data.pop("type")

                line_items = payload.get("line_items", [])

                for i in range(len(line_items)):
                    if "item_ledger_id" in line_items[i].keys():
                        line_items[i]["product_id"] = int(line_items[i].pop("item_ledger_id"))
                    if "quantity" in line_items[i].keys():
                        line_items[i]["quantity"] = int(line_items[i].pop("quantity"))
                    if "price" in line_items[i].keys():
                        line_items[i]["price_unit"] = int(line_items[i].pop("price"))
                    if "tax_id" in line_items[i].keys():
                        line_items[i]["tax_ids"] = [int(line_items[i].pop("tax_id"))]

                    if "description" in line_items[i].keys():
                        line_items[i].pop("description")
                    if "account_id" in line_items[i].keys():
                        line_items[i].pop("account_id")
                    if "tax_type" in line_items[i].keys():
                        line_items[i].pop("tax_type")
                    if "type" in line_items[i].keys():
                        line_items[i].pop("type")
                    if "unit_of_measure" in line_items[i].keys():
                        line_items[i].pop("unit_of_measure")

                # if line_items has data in it
                if bool(line_items):
                    data.update({"invoice_line_ids": line_items})

            # If client type is MS_DYNAMICS
            elif super().get_client_type() == super().MS_DYNAMICS:
                data.pop("type")
                data.update({
                    "customerNumber": f'{payload.get("customer_ledger_id", "")}',
                    "due_date": f'{payload.get("due_date", (datetime.now() + timedelta(days=1)).strftime("%Y-%m-%d"))}',
                })

                line_items = payload.get("line_items", [])

                for i in range(len(line_items)):
                    if "item_ledger_id" in line_items[i].keys():
                        line_items[i]["item_id"] = line_items[i].pop(
                            "item_ledger_id"
                        )
                    if "quantity" in line_items[i].keys():
                        line_items[i]["quantity"] = line_items[i].pop("quantity")
                    if "price" in line_items[i].keys():
                        line_items[i]["unit_price"] = line_items[i].pop("price")
                    if "type" in line_items[i].keys():
                        line_items[i]["type"] = line_items[i].pop("type")
                    if "unit_of_measure" in line_items[i].keys():
                        line_items[i]["unit_of_measure"] = line_items[i].pop("unit_of_measure")

                    if "account_id" in line_items[i].keys():
                        line_items[i].pop("account_id")
                    if "description" in line_items[i].keys():
                        line_items[i].pop("description")
                    if "tax_type" in line_items[i].keys():
                        line_items[i].pop("tax_type")
                    if "tax_id" in line_items[i].keys():
                        line_items[i].pop("tax_id")

                # if line_items has data in it
                if bool(line_items):
                    data.update({"create_customer_invoice_details": line_items})
                    
            data.update(additional_properties)

            return data

        elif operation == super().READ:

            payload = super().response()

            data = payload

            # confirms if a single object was read from the database
            if isinstance(payload, dict):
                if "resource" in payload.keys():
                    data = payload.get("resource", [])
                elif "invoices" in payload.keys():
                    data = payload.get("invoices", [])

            # confirms if a single object was read from the database
            if isinstance(data, dict):
                data = [data]

            # confirms if data is a list
            if isinstance(data, list):

                if len(data) > 0:
                    for i in range(len(data)):
                        if "total_amount" in data[i].keys():
                            data[i]["amount"] = data[i].pop("total_amount")

            if isinstance(payload, dict):
                if "invoices" in payload.keys():
                    payload["invoices"] = data
                elif "resource" in payload.keys():
                    payload["resource"] = data

            super().set_response(payload)

            return self



from erp_sync.Resources.resource import Resource

# This class is used to handle the Customer entity/resource 
class Customer(Resource):

    # This class lacks a constructor because when it is initialized on the 'Entry Class' Clients Class,
    # the clients class set's its default values by calling it's parents methods.

    # This was done so, so as to ensure that the client class can have more control over the entity 'Sub Classes' i.e. Customer

    urls = {}

    # overrides the method below that's used to set the type of erp a interacting with
    def set_client_type(self, client_type_id):
        super().set_client_type(client_type_id)
        return self

    # overrides the method below that's used to set the ID of the erp a user is interacting with
    def set_client_id(self, client_id):
        super().set_client_id(client_id)
        self._set_urls()
        return self
        
    # overrides the method below that's used to set the ID of the company in the erp a user is interacting with
    def set_company_id(self, company_id):
        super().set_company_id(company_id)
        self._set_urls()
        return self

    # set the URLs to be used to execute the CRUD operations
    # this method is dependent on the company id, hence call this method after a company id is set or changed

    # because this class has no constructor and we need these CRUD URLs, we have to call this method when executing
    # any CRUD method, as well as after changing the any CRUD URL endpoint
    def _set_urls(self):

        self.urls = {
            "new": f"/companies/{super().get_company_id()}/customers",
            "edit": f"/companies/{super().get_company_id()}/customers",
            "read": f"/companies/{super().get_company_id()}/customers",
            "delete": f"/companies/{super().get_company_id()}/customers",
            "import": f"/companies/{super().get_company_id()}/import_customers"
        }

        # once the URLs are set pass them to the super class set urls method.
        # this is done so because it is the super class that will execute the CRDU requests
        super().set_urls(self.urls)

        return self

    # a method to update an entry
    def edit(self, ledger_id=None, payload=None, method='PUT', endpoint=None):

        # first set the CRUD URLs
        self._set_urls()

        self.urls["edit"] = f'{self.urls["edit"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().edit(payload, method, endpoint)

    # a method to read entries
    def read(self, customer_id=None, payload=None, method='GET', endpoint=None):

        self._set_urls()

        if customer_id is not None:
            self.urls["read"] = f'{self.urls["read"]}/{customer_id}'
            super().set_urls(self.urls)

        return super().read(payload, method, endpoint)

    # a method to delete an entry
    def delete(self, ledger_id=None, payload=None, method='DELETE', endpoint=None):

        self._set_urls()

        self.urls["delete"] = f'/{self.urls["delete"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().delete(payload, method, endpoint)

    # a method to import entries
    def import_data(self, ledger_id=None, payload=None, method='GET', endpoint=None):

        self._set_urls()

        if ledger_id is not None:
            self.urls["import"] = f'{self.urls["import"]}/{ledger_id}'
            super().set_urls(self.urls)

        return super().import_data(payload, method, endpoint)

    # a custom entity specific method to import vendors
    def import_vendors(self, ledger_id=None, payload=None, method='GET', endpoint=None):

        endpoint = f"/companies/{super().get_company_id()}/import_vendors"

        if ledger_id is not None:
            endpoint = f'{endpoint}/{ledger_id}'

        return super().import_data(payload, method, endpoint)

    # This overidden method is used to return the default payload expected by this class
    def payload(self):

        data = {
            "type": "<Enter either customer or vendor>",
            "title": "Mr",
                    "first_name": "<Enter first name>",
                    "last_name": "<Enter last name>",
                    "email": "<Enter email address>",
                    "phone": "<Enter phone number>",
                    "mobile": "<Enter mobile number>",
                    "designation": "<Enter designation e.g. Sales Executive>",
                    "department": "<Enter department e.g. Sales and Marketing>"
        }

        if super().get_client_type() == super().XERO:

            data["type"] = "<Enter either True (for vendor) or False (for customer)>"

        data.update({
            "additional_properties": {
                "help": "Optional data are passed inside this object, the format should be specific to the ERP. View ERP documentation for more details, below are examples of optional data",
            }
        })

        return data

    # This overidden method is used to convert the standardized CRUD operations payloads. It serves two functions
    # 1. It will take the standardized payloads and return the expected client/erp specific payload 
    # 2. It will get responses from READ operations and standardize the responses
    def serialize(self, payload=None, operation=None):

        data = {}

        if operation is None:
            return "Specify the operation: Resource.READ, Resource.NEW or Resource.UPDATE"

        if operation == super().NEW or operation == super().UPDATE:

            additional_properties = payload.get("additional_properties", {})

            # If client type is Quickbooks Online
            if super().get_client_type() == super().QBO:
                
                data.update({
                    "type": payload.get("type", "Customer")
                })

                if 'title' in payload.keys():
                    data.update({
                        "Title": payload.get("title", "")
                    })

                if 'first_name' in payload.keys() and 'last_name' in payload.keys():
                    data.update({
                        "DisplayName": f'{payload.get("first_name","")} {payload.get("last_name","")}',
                        "GivenName": payload.get("first_name", ""),
                        "MiddleName": payload.get("last_name", ""),
                        "FamilyName": payload.get("last_name", "")
                    })

                if 'email' in payload.keys():
                    data.update({
                        "PrimaryEmailAddr": {
                            "Address": payload.get("email", ""),
                        }
                    })

                if 'phone' in payload.keys():
                    data.update({
                        "PrimaryPhone": {
                            "FreeFormNumber": payload.get("phone", ""),
                        }
                    })

            # If client type is SAP
            elif super().get_client_type() == super().SAP:
                bp_information = {
                    "CardType": "CUSTOMER",
                }
                if 'first_name' in payload.keys() and 'last_name' in payload.keys():
                    bp_information.update({
                        "CardName": f'{payload.get("first_name","")} {payload.get("last_name","")}'
                    })

                if 'phone' in payload.keys():
                    bp_information.update({
                        "Telephone1": payload.get("phone", ""),
                        "Telephone2": payload.get("phone", "")
                    })

                if 'mobile' in payload.keys():
                    bp_information.update({
                        "MobilePhone": payload.get("mobile", "")
                    })

                if 'email' in payload.keys():
                    bp_information.update({
                        "Email": payload.get("email", "")
                    })

                if 'tax_pin' in payload.keys():
                    bp_information.update({
                        "KRAPIN": payload.get("tax_pin", "")
                    })

                if 'customer_code' in payload.keys():
                    bp_information.update({
                        "CardCode": payload.get("customer_code", "")
                    })

                if 'GroupCode' in additional_properties.keys():
                    bp_information.update({
                        "GroupCode": additional_properties.get("GroupCode", {})
                    })

                    additional_properties.pop("GroupCode")

                if 'Fax' in additional_properties.keys():
                    bp_information.update({
                        "Fax": additional_properties.get("Fax", {})
                    })

                    additional_properties.pop("Fax")

                if 'phone2' in additional_properties.keys():
                    bp_information.update({
                        "Telephone2": additional_properties.get("phone2", {})
                    })

                    additional_properties.pop("phone2")

                data.update({
                    "BPInformation": bp_information
                })
            # If client type is ZOHO
            elif super().get_client_type() == super().ZOHO:

                contact_persons = {}

                data.update({
                    "contact_type": "customer",
                    "customer_sub_type": "business",
                })

                if 'title' in payload.keys():
                    data.update({
                        "salutation": payload.get("title", "")
                    })

                if 'first_name' in payload.keys() and 'last_name' in payload.keys():
                    data.update({
                        "contact_name": f'{payload.get("first_name","")} {payload.get("last_name","")}'
                    })

                    contact_persons.update({
                        "first_name": payload.get("first_name", ""),
                        "last_name": payload.get("last_name", "")
                    })

                if 'email' in payload.keys():
                    contact_persons.update({
                        "email": payload.get("email", "")
                    })

                if 'phone' in payload.keys():
                    contact_persons.update({
                        "phone": payload.get("phone", "")
                    })

                if 'mobile' in payload.keys():
                    contact_persons.update({
                        "mobile": payload.get("mobile", "")
                    })

                if 'designation' in payload.keys():
                    contact_persons.update({
                        "designation": payload.get("designation", "")
                    })

                if 'department' in payload.keys():
                    contact_persons.update({
                        "department": payload.get("department", "")
                    })

                if 'is_primary_contact' in payload.keys():
                    contact_persons.update({
                        "is_primary_contact": True
                    })

                if 'enable_portal' in payload.keys():
                    contact_persons.update({
                        "enable_portal": True
                    })

                data.update({
                    "contact_persons": [contact_persons]
                })

            # If client type is XERO
            elif super().get_client_type() == super().XERO:

                contacts = {"IsSupplier": False}

                if 'first_name' in payload.keys() and 'last_name' in payload.keys():
                    contacts.update({
                        "Name": f'{payload.get("first_name","")} {payload.get("last_name","")}',
                        "FirstName": payload.get("first_name", ""),
                        "LastName": payload.get("last_name", ""),
                        "ContactStatus": additional_properties.get("ContactStatus", "ACTIVE"),
                        "IsSupplier": additional_properties.get("IsSupplier", False)
                    })

                    if 'ContactStatus' in additional_properties.keys():
                        additional_properties.pop("ContactStatus")

                    if 'IsSupplier' in additional_properties.keys():
                        additional_properties.pop("IsSupplier")

                if 'email' in payload.keys():
                    contacts.update({
                        "EmailAddress": payload.get("email", "")
                    })

                if 'phone' in payload.keys():
                    contacts.update({
                        "Phones": [
                            {
                                "PhoneType": additional_properties.get("PhoneType", "MOBILE"),
                                "PhoneNumber": payload.get("phone", ""),
                                "PhoneAreaCode": additional_properties.get("PhoneAreaCode", "254")
                            }
                        ]
                    })

                    if 'PhoneType' in additional_properties.keys():
                        additional_properties.pop("PhoneType")

                    if 'PhoneAreaCode' in additional_properties.keys():
                        additional_properties.pop("PhoneAreaCode")

                if bool(contacts):
                    data["Contacts"] = [contacts]

            # If client type is ODOO
            elif super().get_client_type() == super().ODOO:
                data.update({
                    "name": f'{payload.get("first_name", "")} {payload.get("last_name", "")}',
                    "display_name": f'{payload.get("first_name", "")} {payload.get("last_name", "")}',
                    "email": f'{payload.get("email", "")}',
                    "phone": f'{payload.get("phone", "")}',
                    "active":True
                })
            # If client type is MS_DYNAMICS
            elif super().get_client_type() == super().MS_DYNAMICS:
                data.update({
                    "name": f'{payload.get("first_name", "")} {payload.get("last_name", "")}',
                    "email": f'{payload.get("email", "")}',
                    "city": f'{payload.get("city", "")}',
                    "phoneNumber": f'{payload.get("phone", "")}',
                    "address": f'{payload.get("address", "")}',
                })

            data.update(additional_properties)

            return data

        elif operation == super().READ:

            payload = super().response()

            data = payload

            if 'customers' in payload.keys():
                data = payload.get("customers", [])
            elif 'resource' in payload.keys():
                data = payload.get("resource", [])

            # confirms if a single object was read from the database
            if isinstance(data, dict):
                data = [data]

            # confirms if data is a list
            if isinstance(data, list):

                if len(data) > 0:

                    for i in range(len(data)):
                        if 'family_name' in data[i].keys():
                            data[i]['first_name'] = data[i].pop('family_name')

                        if 'given_name' in data[i].keys():
                            data[i]['last_name'] = data[i].pop('given_name')

            if 'customers' in payload.keys():
                payload["customers"] = data
            elif 'resource' in payload.keys():
                payload["resource"] = data

            super().set_response(payload)

            return self



from erp_sync.Resources.resource import Resource


class Vendors(Resource):

    urls = {}

    def set_client_id(self, client_id):
        super().set_client_id(client_id)
        self._set_urls()
        return self

    def set_company_id(self, company_id):
        super().set_company_id(company_id)
        self._set_urls()
        return self

    def _set_urls(self):

        self.urls = {
            "new": f"/companies/{super().get_company_id()}/vendors",
            "read": f"/companies/{super().get_company_id()}/vendors",
            "edit": f"/companies/{super().get_company_id()}/vendors",
            "delete": f"/companies/{super().get_company_id()}/vendors",
            "import": f"/companies/{super().get_company_id()}/import_vendors"
        }

        super().set_urls(self.urls)

        return self
    
    def new(self, payload = None, method='POST',endpoint=None):

        self._set_urls()

        if super().get_client_type() in [super().MS_DYNAMICS,super().ZOHO,super().ODOO,super().QBO,super().XERO]:
            self.urls["new"] = f"/companies/{super().get_company_id()}/customers?type=Vendor"
            super().set_urls(self.urls)
            
        return super().new(payload, method, endpoint)

    def read(self, vendor_id=None, payload=None, method='GET', endpoint=None):

        self._set_urls()

        if vendor_id is not None:
            self.urls["read"] = f'{self.urls["read"]}/{vendor_id}'
            super().set_urls(self.urls)

        return super().read(payload, method, endpoint)

    def edit(self, ledger_id=None, payload=None, method='PUT', endpoint=None):

        self._set_urls()

        self.urls["edit"] = f'{self.urls["edit"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().edit(payload, method, endpoint)

    def delete(self, ledger_id=None, payload=None, method='DELETE', endpoint=None):

        self._set_urls()

        self.urls["delete"] = f'{self.urls["delete"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().delete(payload, method, endpoint)

    def import_data(self, ledger_id=None, payload=None, method='GET', endpoint=None):

        self._set_urls()

        if ledger_id is not None:
            self.urls["import"] = f'{self.urls["import"]}/{ledger_id}'

            super().set_urls(self.urls)

        return super().import_data(payload, method, endpoint)

    def payload(self):

        data = {
            "name": "Noella Bergstrom CPA",
            "email": "jon@kerluke-tillman.info",
            "city": "Nairobi",
            "phone_number": "2433981830",
            "address": "Flamingo Towers"
        }

        return data

    def serialize(self, payload=None, operation=None):

        data = {}

        if operation is None:
            return "Specify the operation: Resource.READ, Resource.NEW or Resource.UPDATE"

        if operation == super().NEW or operation == super().UPDATE:

            additional_properties = payload.get("additional_properties", {})

            # If client type is Quickbooks Online or MS_DYNAMICS
            if super().get_client_type() == super().MS_DYNAMICS:
                data.update({
                    "type": "Vendor",
                    "name": f'{payload.get("first_name", "")} {payload.get("last_name", "")}',
                    "email": f'{payload.get("email", "")}',
                    "city": f'{payload.get("city", "")}',
                    "phoneNumber": f'{payload.get("phone", "")}',
                    "address": f'{payload.get("address", "")}',
                })
            elif super().get_client_type() == super().QBO:
                data.update({
                    "DisplayName": f'{payload.get("first_name", "")} {payload.get("last_name", "")}',
                    "Title": payload.get("title", ""),
                    "GivenName": payload.get("first_name", ""),
                    "MiddleName": payload.get("last_name", ""),
                    "FamilyName": payload.get("last_name", ""),
                    "PrimaryEmailAddr": {
                        "Address": payload.get("email", "")
                    },
                    "PrimaryPhone": {
                        "FreeFormNumber": payload.get("phone", "")
                    },
                    "type": "Vendor"
                })
            elif super().get_client_type() == super().ZOHO:
                contact_persons = {}

                data.update({
                    "contact_type": "vendor",
                    "customer_sub_type": "business",
                })

                if 'title' in payload.keys():
                    data.update({
                        "salutation": payload.get("title", "")
                    })

                if 'first_name' in payload.keys() and 'last_name' in payload.keys():
                    data.update({
                        "contact_name": f'{payload.get("first_name","")} {payload.get("last_name","")}'
                    })

                    contact_persons.update({
                        "first_name": payload.get("first_name", ""),
                        "last_name": payload.get("last_name", "")
                    })

                if 'email' in payload.keys():
                    contact_persons.update({
                        "email": payload.get("email", "")
                    })

                if 'phone' in payload.keys():
                    contact_persons.update({
                        "phone": payload.get("phone", "")
                    })

                if 'mobile' in payload.keys():
                    contact_persons.update({
                        "mobile": payload.get("mobile", "")
                    })

                if 'designation' in payload.keys():
                    contact_persons.update({
                        "designation": payload.get("designation", "")
                    })

                if 'department' in payload.keys():
                    contact_persons.update({
                        "department": payload.get("department", "")
                    })

                if 'is_primary_contact' in payload.keys():
                    contact_persons.update({
                        "is_primary_contact": True
                    })

                if 'enable_portal' in payload.keys():
                    contact_persons.update({
                        "enable_portal": True
                    })

                data.update({
                    "contact_persons": [contact_persons]
                })           

            elif super().get_client_type() == super().ODOO:
                data.update({
                    "name": f'{payload.get("first_name", "")} {payload.get("last_name", "")}',
                    "display_name": f'{payload.get("first_name", "")} {payload.get("last_name", "")}',
                    "email": f'{payload.get("email", "")}',
                    "phone": f'{payload.get("phone", "")}',
                    "active":True
                })
            
            # If client type is XERO
            elif super().get_client_type() == super().XERO:

                contacts = {"IsSupplier": True}

                if 'first_name' in payload.keys() and 'last_name' in payload.keys():
                    contacts.update({
                        "Name": f'{payload.get("first_name","")} {payload.get("last_name","")}',
                        "FirstName": payload.get("first_name", ""),
                        "LastName": payload.get("last_name", ""),
                        "ContactStatus": additional_properties.get("ContactStatus", "ACTIVE")
                    })

                    if 'ContactStatus' in additional_properties.keys():
                        additional_properties.pop("ContactStatus")

                    if 'IsSupplier' in additional_properties.keys():
                        additional_properties.pop("IsSupplier")

                if 'email' in payload.keys():
                    contacts.update({
                        "EmailAddress": payload.get("email", "")
                    })

                if 'phone' in payload.keys():
                    contacts.update({
                        "Phones": [
                            {
                                "PhoneType": additional_properties.get("PhoneType", "MOBILE"),
                                "PhoneNumber": payload.get("phone", ""),
                                "PhoneAreaCode": additional_properties.get("PhoneAreaCode", "254")
                            }
                        ]
                    })

                    if 'PhoneType' in additional_properties.keys():
                        additional_properties.pop("PhoneType")

                    if 'PhoneAreaCode' in additional_properties.keys():
                        additional_properties.pop("PhoneAreaCode")

                if bool(contacts):
                    data["Contacts"] = [contacts]

            data.update(additional_properties)

            return data

        elif operation == super().READ:

            payload = super().response()

            # confirms if a single object was read from the database
            if isinstance(payload, dict):
                if 'resource' in payload.keys():
                    data = payload.get("resource", [])

                    # confirms if a single object was read from the database
                    if isinstance(data, dict):
                        data = [data]
                else:
                    data = [payload]

            elif isinstance(payload, list):
                data = payload

            # No serialization will be done here as the data returned is already in the
            # expected serialized format

            super().set_response(data)

            return self



from erp_sync.Resources.resource import Resource


class Banks(Resource):

    urls = {}

    def set_client_id(self, client_id):
        super().set_client_id(client_id)
        self._set_urls()
        return self

    def set_company_id(self, company_id):
        super().set_company_id(company_id)
        self._set_urls()
        return self

    def _set_urls(self):

        self.urls = {
            "new": f"/companies/{super().get_company_id()}/banks",
            "edit": f"/companies/{super().get_company_id()}/banks",
            "read": f"/companies/{super().get_company_id()}/banks",
            "delete": f"/companies/{super().get_company_id()}/banks",
            "import": f"/companies/{super().get_company_id()}/import_banks"
        }

        super().set_urls(self.urls)

        return self

    def read(self, bank_id=None, payload=None, method='GET', endpoint=None):

        self._set_urls()

        if bank_id is not None:
            self.urls["read"] = f'{self.urls["read"]}/{bank_id}'
            super().set_urls(self.urls)

        return super().read(payload, method, endpoint)

    def edit(self, ledger_id=None, payload=None, method='PUT', endpoint=None):

        self._set_urls()

        self.urls["edit"] = f'{self.urls["edit"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().edit(payload, method, endpoint)

    def delete(self, ledger_id=None, payload=None, method='DELETE', endpoint=None):

        self._set_urls()

        self.urls["delete"] = f'{self.urls["delete"]}/{ledger_id}'

        super().set_urls(self.urls)

        return super().delete(payload, method, endpoint)

    def import_data(self, ledger_id=None, payload=None, method='GET', endpoint=None):

        self._set_urls()

        if ledger_id is not None:
            self.urls["import"] = f'{self.urls["import"]}/{ledger_id}'
            super().set_urls(self.urls)

        return super().import_data(payload, method, endpoint)

    def payload(self):

        data = {}

        # If client type is Quickbooks Online
        if super().get_client_type() == super().ODOO:

            data = {
                "name": "Test Bank"
            }

        return data

    def serialize(self, payload=None, operation=None):

        data = {}

        if operation is None:
            return "Specify the operation: Resource.READ, Resource.NEW or Resource.UPDATE"

        if operation == super().NEW or operation == super().UPDATE:

            # If client type is ODOO
            if super().get_client_type() == super().ODOO:

                if 'name' in payload.keys():
                    data.update({
                        "name": payload.get("name", "")
                    })

            data.update(payload.get("additional_properties", {}))

            return data

        elif operation == super().READ:

            super().set_response(payload)

            return self
